@@ -1,11 +1,10 @@
-use std::cmp::Ordering;
 use std::collections::{HashMap, BTreeMap};
 
+use itertools::Itertools;
 use log::{self, log_enabled, trace};
-use num_traits::Zero;
 
 use crate::broker_statement::{BrokerStatement, StockSource, StockSellType};
-use crate::config::{PortfolioConfig, PerformanceMergingConfig};
+use crate::config::PortfolioConfig;
 use crate::core::{EmptyResult, GenericResult};
 use crate::currency::Cash;
 use crate::currency::converter::CurrencyConverter;
@@ -15,8 +14,10 @@ use crate::taxes::{NetTax, NetTaxCalculator, NetLtoDeduction, NetLtoDeductionCal
 use crate::time::{self, Date, DateOptTime};
 use crate::types::Decimal;
 
+use super::config::PerformanceMergingConfig;
 use super::deposit_emulator::{Transaction, InterestPeriod};
 use super::deposit_performance;
+use super::instrument_view::InstrumentDepositView;
 use super::portfolio_analysis::{
     PortfolioPerformanceAnalysis, InstrumentPerformanceAnalysis, IncomeStructure};
 
@@ -27,12 +28,12 @@ pub struct PortfolioPerformanceAnalyser<'a> {
     country: &'a Country,
     currency: &'a str,
     converter: &'a CurrencyConverter,
-    merge_performance: Option<&'a PerformanceMergingConfig>,
     include_closed_positions: bool,
+    performance_merging_config: Option<PerformanceMergingConfig>,
 
     transactions: Vec<Transaction>,
     income_structure: IncomeStructure,
-    instruments: Option<BTreeMap<String, StockDepositView>>,
+    instruments: Option<BTreeMap<String, InstrumentDepositView>>,
     net_lto_calc: NetLtoDeductionCalculator,
     current_assets: Decimal,
 }
@@ -40,15 +41,15 @@ pub struct PortfolioPerformanceAnalyser<'a> {
 impl <'a> PortfolioPerformanceAnalyser<'a> {
     pub fn new(
         country: &'a Country, currency: &'a str, converter: &'a CurrencyConverter,
-        merge_performance: Option<&'a PerformanceMergingConfig>, include_closed_positions: bool,
+        include_closed_positions: bool,
     ) -> PortfolioPerformanceAnalyser<'a> {
         PortfolioPerformanceAnalyser {
             today: time::today(),
             country,
             currency,
             converter,
-            merge_performance,
             include_closed_positions,
+            performance_merging_config: None,
 
             transactions: Vec::new(),
             income_structure: Default::default(),
@@ -58,7 +59,17 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
         }
     }
 
-    pub fn add(&mut self, portfolio: &PortfolioConfig, statement: &BrokerStatement) -> EmptyResult {
+    pub fn add(
+        &mut self, portfolio: &PortfolioConfig, statement: &BrokerStatement,
+        merge_performance: PerformanceMergingConfig,
+    ) -> EmptyResult {
+        self.performance_merging_config.replace(merge_performance);
+        let result = self.add_inner(portfolio, statement);
+        self.performance_merging_config.take();
+        result
+    }
+
+    fn add_inner(&mut self, portfolio: &PortfolioConfig, statement: &BrokerStatement) -> EmptyResult {
         // Assume that the caller has simulated sellout and just check it here
         if !statement.open_positions.is_empty() {
             return Err!(
@@ -70,6 +81,7 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
         self.process_positions(statement, portfolio)?;
         self.process_dividends(statement, portfolio)?;
         self.process_interest(statement, portfolio)?;
+        self.process_fees(statement)?;
         self.process_tax_agent_withholdings(statement)?;
         self.process_tax_deductions(portfolio)?;
         self.process_cash_assets(statement)?;
@@ -108,7 +120,7 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
     }
 
     fn analyse_instrument_performance(
-        &mut self, symbol: &str, mut deposit_view: StockDepositView
+        &mut self, symbol: &str, mut deposit_view: InstrumentDepositView
     ) -> GenericResult<InstrumentPerformanceAnalysis> {
         deposit_view.transactions.sort_by_key(|transaction| transaction.date);
 
@@ -174,71 +186,16 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
     }
 
     fn calculate_open_position_periods(&mut self) -> EmptyResult {
-        struct OpenPosition {
-            start_date: Date,
-            quantity: Decimal,
-        }
-
         trace!("Open positions periods:");
 
         for (symbol, deposit_view) in self.instruments.as_mut().unwrap() {
-            if deposit_view.trades.is_empty() {
-                return Err!("Got an unexpected transaction for {} which has no trades", symbol)
-            }
-
-            let mut open_position = None;
-
-            for (&date, &quantity) in &deposit_view.trades {
-                let current = open_position.get_or_insert_with(|| {
-                    OpenPosition {
-                        start_date: date,
-                        quantity: dec!(0),
-                    }
-                });
-                current.quantity += quantity;
-
-                match current.quantity.cmp(&Decimal::zero()) {
-                    Ordering::Greater => continue,
-                    Ordering::Equal => {},
-                    Ordering::Less => {
-                        return Err!(
-                            "Error while processing {} sell operations: Got a negative balance on {}",
-                            symbol, formatting::format_date(date));
-                    }
-                };
-
-                let start_date = current.start_date;
-                let end_date = if date == start_date {
-                    date.succ()
-                } else {
-                    date
-                };
-
-                match deposit_view.interest_periods.last_mut() {
-                    Some(ref mut period) if period.end >= start_date => {
-                        assert_eq!(period.end, start_date);
-                        assert!(period.end < end_date);
-                        period.end = end_date;
-                    },
-                    _ => deposit_view.interest_periods.push(InterestPeriod::new(start_date, end_date)),
-                };
-
-                open_position = None;
-            }
-
-            if open_position.is_some() {
-                return Err!(
-                    "The portfolio contains unsold {} stocks when sellout simulation is expected",
-                    symbol);
-            }
-            assert!(!deposit_view.interest_periods.is_empty());
+            deposit_view.calculate_open_position_periods()?;
 
             if log_enabled!(log::Level::Trace) {
                 let periods = deposit_view.interest_periods.iter()
                     .map(|period| format!(
                         "{} - {}", formatting::format_date(period.start),
                         formatting::format_date(period.end)))
-                    .collect::<Vec<_>>()
                     .join(", ");
 
                 trace!("* {}: {}", symbol, periods);
@@ -287,13 +244,13 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
                     self.income_structure.commissions += commission;
 
                     let deposit_view = self.get_deposit_view(&trade.symbol);
-                    deposit_view.trade(trade.conclusion_time, quantity);
+                    deposit_view.trade(&portfolio.name, &trade.symbol, trade.conclusion_time, quantity);
                     deposit_view.transaction(trade.conclusion_time, volume);
                     deposit_view.transaction(trade.conclusion_time, commission);
                 },
                 StockSource::CorporateAction => {
-                    let deposit_view = self.get_deposit_view(&trade.symbol);
-                    deposit_view.trade(trade.conclusion_time, quantity);
+                    self.get_deposit_view(&trade.symbol).trade(
+                        &portfolio.name, &trade.symbol, trade.conclusion_time, quantity);
                 },
             };
         }
@@ -315,7 +272,7 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
                     {
                         let deposit_view = self.get_deposit_view(&trade.symbol);
 
-                        deposit_view.trade(trade.conclusion_time, -quantity);
+                        deposit_view.trade(&portfolio.name, &trade.symbol, trade.conclusion_time, -quantity);
                         deposit_view.transaction(trade.conclusion_time, -volume);
                         deposit_view.transaction(trade.conclusion_time, commission);
 
@@ -348,8 +305,8 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
                         &lto_deductibles, trade.emulation);
                 },
                 StockSellType::CorporateAction => {
-                    let deposit_view = self.get_deposit_view(&trade.symbol);
-                    deposit_view.trade(trade.conclusion_time, -quantity);
+                    self.get_deposit_view(&trade.symbol).trade(
+                        &portfolio.name, &trade.symbol, trade.conclusion_time, -quantity);
                 },
             };
         }
@@ -372,12 +329,12 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
             if let Some(amount) = self.map_tax_to_deposit_amount(tax_payment_date, tax_to_pay)? {
                 trace!("* Stock selling {} tax: {}", formatting::format_date(tax_payment_date), amount);
                 self.transaction(tax_payment_date, amount);
-                self.income_structure.taxes += amount;
+                self.income_structure.trading_taxes += amount;
             }
 
             if let Some(amount) = self.map_tax_to_deposit_amount(tax_payment_date, tax_deduction)? {
                 trace!("* {} tax deduction: {}", formatting::format_date(tax_payment_date), amount);
-                self.income_structure.tax_deductions += amount;
+                self.income_structure.trading_tax_deductions += amount;
             }
 
             self.net_lto_calc.add_applied_deduction(tax_year, lto_deduction.amount, lto_loss.amount);
@@ -406,7 +363,7 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
 
                 self.get_deposit_view(&dividend.issuer).transaction(tax_payment_date.into(), amount);
                 self.transaction(tax_payment_date, amount);
-                self.income_structure.taxes += amount;
+                self.income_structure.dividend_taxes += amount;
             }
         }
 
@@ -427,13 +384,22 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
                        formatting::format_date(tax_payment_date), amount);
 
                 self.transaction(tax_payment_date, amount);
-                self.income_structure.taxes += amount;
+                self.income_structure.interest_taxes += amount;
             }
         }
 
         Ok(())
     }
 
+    fn process_fees(&mut self, statement: &BrokerStatement) -> EmptyResult {
+        for fee in &statement.fees {
+            self.income_structure.commissions += self.converter.convert_to(
+                fee.date, fee.amount, self.currency)?;
+        }
+
+        Ok(())
+    }
+
     fn process_tax_agent_withholdings(&mut self, statement: &BrokerStatement) -> EmptyResult {
         for withholding in &statement.tax_agent_withholdings {
             let amount = self.converter.convert_to(withholding.date, withholding.amount, self.currency)?;
@@ -449,7 +415,7 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
             let amount = self.converter.convert(self.country.currency, self.currency, date, amount)?;
             trace!("* Tax deduction {}: {}", formatting::format_date(date), amount);
             self.transaction(date, -amount);
-            self.income_structure.tax_deductions += amount;
+            self.income_structure.additional_tax_deductions += amount;
         }
 
         Ok(())
@@ -461,23 +427,11 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
         Ok(())
     }
 
-    fn get_deposit_view(&mut self, symbol: &str) -> &mut StockDepositView {
-        // FIXME(konishchev): A temporary workaround for merge performance conflicts with stock split
-        if let Some(merge_performance) = self.merge_performance.as_ref() {
-            for (master_symbol, slave_symbols) in merge_performance.iter() {
-                for slave_symbol in slave_symbols {
-                    if symbol == slave_symbol {
-                        return self.instruments.as_mut().unwrap()
-                            .entry(master_symbol.to_owned())
-                            .or_insert_with(StockDepositView::new)
-                    }
-                }
-            }
-        }
-
+    fn get_deposit_view(&mut self, symbol: &str) -> &mut InstrumentDepositView {
+        let mapped_symbol = self.performance_merging_config.as_ref().unwrap().map(symbol);
         self.instruments.as_mut().unwrap()
-            .entry(symbol.to_owned())
-            .or_insert_with(StockDepositView::new)
+            .entry(mapped_symbol.to_owned())
+            .or_insert_with(|| InstrumentDepositView::new(mapped_symbol))
     }
 
     fn transaction(&mut self, date: Date, amount: Decimal) {
@@ -503,40 +457,6 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
     }
 }
 
-struct StockDepositView {
-    name: Option<String>,
-    trades: BTreeMap<Date, Decimal>,
-    transactions: Vec<Transaction>,
-    interest_periods: Vec<InterestPeriod>,
-    closed: bool,
-}
-
-impl StockDepositView {
-    fn new() -> StockDepositView {
-        StockDepositView {
-            name: None,
-            trades: BTreeMap::new(),
-            transactions: Vec::new(),
-            interest_periods: Vec::new(),
-            closed: true,
-        }
-    }
-
-    fn trade(&mut self, time: DateOptTime, quantity: Decimal) {
-        self.trades.entry(time.date)
-            .and_modify(|total| *total += quantity)
-            .or_insert(quantity);
-    }
-
-    fn transaction(&mut self, time: DateOptTime, amount: Decimal) {
-        // Some assets can be acquired for free due to corporate actions or other non-trading
-        // operations.
-        if !amount.is_zero() {
-            self.transactions.push(Transaction::new(time.date, amount))
-        }
-    }
-}
-
 fn get_total_activity_duration(periods: &[InterestPeriod]) -> u32 {
     periods.iter().map(InterestPeriod::days).sum()
 }
\ No newline at end of file
